/*
 * fi_download_protocol.h
 *
 *  Created on: Jan 25, 2016
 *      Author: eric
 */

#ifndef FI_DOWNLOAD_PROTOCOL_H_
#define FI_DOWNLOAD_PROTOCOL_H_

#include <vector>

#include "fi_streaming_session.h"
#include "fi_streaming_protocol.h"

class BufferQueue;

/**
 * @brief StreamingProtocol tailored for image file download
 *
 * Clients can use this WebSocket protocol to receive streams of image data
 * suitable to save onto their local storage as images in a number of standard
 * image file formats. The formats are:
 *     - JPEG compressed
 *     - TIFF, RGB uncompressed
 *     - BMP, RGB uncompressed
 *     - DNG, raw (one color channel per pixel) uncompressed
 */
class fiWSSDownloadProtocol: public StreamingProtocol
{
public:
    fiWSSDownloadProtocol(struct lws_protocols* p);
    virtual ~fiWSSDownloadProtocol();

    virtual void release_image_memory_buffer( void* shmem );

    /**
     * @brief Accept at most one Download connection
     *
     * The Download protocol only makes sense for one client at a time.
     * We use the libwebsockets LWS_CALLBACK_FILTER_PROTOCOL_CONNECTION
     * callback as the opportunity to enforce this.
     */
    virtual int accept(struct lws *wsi, WssSession** user);
    virtual int established(struct lws *wsi, WssSession** user);
    virtual int closed(struct lws *wsi, WssSession** user);
    virtual int destroy(struct lws *wsi, WssSession* user);
    virtual int writable(struct lws *wsi, WssSession* user);
    virtual int receive(struct lws *wsi, WssSession* s, uint8_t* in, size_t len);
};

#endif /* FI_DOWNLOAD_PROTOCOL_H_ */
