/*
 * fi_liveview_protocol.h
 *
 *  Created on: Apr 25, 2016
 *      Author: eric
 */

#ifndef FI_LIVEVIEW_PROTOCOL_H_
#define FI_LIVEVIEW_PROTOCOL_H_

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <iterator>

#include "fi_streaming_session.h"
#include "fi_streaming_protocol.h"

/*!
 * @brief A StreamingProtocol tailored for camera's live view stream.
 *
 * Clients can use this WebSocket protocol to receive "live" streams of image
 * data from the camera. Image data is always compressed JPEG.
 */
class fiWSSLiveViewProtocol: public StreamingProtocol
{
public:
    fiWSSLiveViewProtocol(struct lws_protocols* p);
    virtual ~fiWSSLiveViewProtocol();

    uint32_t get_next_id();
    bool has_data();
    void notify( uint32_t reg, uint32_t val );

    virtual int closed(struct lws *wsi, WssSession** user);
    virtual int destroy(struct lws *wsi, WssSession* user);
    virtual int established(struct lws *wsi, WssSession** s);
    virtual void process_consumer();
    virtual int receive(struct lws *wsi, WssSession* s, uint8_t* in, size_t len);
    virtual void release_image_memory_buffer( void* shmem );
    virtual int writable(struct lws *wsi, WssSession* s);

private:
    static const char* PROTOCOL_NAME;
    static const char* JSON_TAG;

    bool cs_is_liveview;	//!< Indicates that the camera is in live view.
    bool cs_is_liveview_update; //!< Indicates a camera state change to a "live view" state.
    uint32_t frame_id;		//!< Synthesize frameID numbers for live view frames.
};

#endif /* FI_LIVEVIEW_PROTOCOL_H_ */
