/*
 * fi_wss_streamer_session.h
 *
 *  Created on: Jan 25, 2016
 *      Author: eric
 */

#ifndef FI_STREAMING_SESSION_H_
#define FI_STREAMING_SESSION_H_

#include <stddef.h>

#include "fi_wss_session.h"

class StreamingProtocol;

/**
 * @brief Base class for all streaming protocol sessions
 *
 * StreamingSession provides some support for streaming protocols
 * that need to manage clients that join or leave streams when the
 * stream is already running. The fiWSSLiveViewProtocol does this,
 * for example.
 */
class StreamingSession : public WssSession
{
public:
    typedef enum
    {
        NONE,
        OFF,
        ON_REQ,
        ON_FRAME,
        ON_IDLE,
        OFF_REQ
    } SessionState;

    StreamingSession(StreamingProtocol* p, struct lws* w);
    virtual ~StreamingSession();

    virtual void scheduleWriteCallback();
    const char *decodeState();
    SessionState getState();
    void setState(SessionState s);

    virtual void setBytesSoFar(int n);
    virtual int getBytesSoFar();
    void set_frame_length( size_t l );
    size_t get_frame_length( void );

    virtual void setTxQueueLinks(StreamingSession* p, StreamingSession* n);

    virtual StreamingSession* getTxQueuePrev();
    virtual void setTxQueuePrev(StreamingSession* p);
    virtual StreamingSession* getTxQueueNext();
    virtual void setTxQueueNext(StreamingSession* n);

private:
    SessionState state;
    bool frameHeaderFinished;
    int bytesSoFar;
    size_t frame_length;
    StreamingSession* txQueueNext;
    StreamingSession* txQueuePrev;
};

#endif /* FI_STREAMING_SESSION_H_ */
