/*
 * fi_wss_session.h
 *
 *  Created on: Jan 27, 2016
 *      Author: eric
 */

#ifndef FI_WSS_SESSION_H_
#define FI_WSS_SESSION_H_

class WssProtocol;

/**
 * @brief Base class for Fastec libwebsockets session
 *
 * Each WebSocket sub-protocol may manage a number of sessions, each with a
 * different client. Sub-protocol implementations may have different needs
 * for sesison management. Each should implement a sub-class of WssSession
 * to capture protocol-specific session management functionality.
 *
 * WssSession provides a wrapper for libwebsockets per-session data as well,
 * so protocols have easy access to it.
 */
class WssSession
{
public:
    /**
     * @brief Constructor
     *
     * @param p Points to the protocol instance to which this session will belong
     * @param w Points to the libwebsockets per-session data for this session
     */
    WssSession(WssProtocol* p, struct lws* w);
    virtual ~WssSession();

    virtual WssProtocol* protocol( void );
    virtual struct lws* get_lws( void );

    /**
     * @brief Discard data queued for transmission to client
     *
     * This is used, for example, during fault recovery when a session's
     * TCP socket must be shut down before all out-bound data has been
     * sent to the client.
     */
    void tx_flush( void );

protected:
    struct lws* wsi;
    WssProtocol* proto;
};

#endif /* FI_WSS_SESSION_H_ */
