/*
 * fi_buffer.cxx
 *
 *  Created on: Jan 25, 2016
 *      Author: eric
 */

#include <stdio.h>
#include <stdlib.h>

#include "fi_buffer.h"

Buffer::Buffer() :
    size(0),
    owner(PRODUCER),
    flags(0),
    id(0)
{
    if (posix_memalign((void**)&data_malloc, BUFF_ALIGN, BUFF_SIZE ) != 0)
    {
        // Alloc error; throw exception?
    }
    data = data_malloc + PRE_BUFF_SIZE;
}

Buffer::~Buffer()
{
    free(data_malloc);
}

void Buffer::prefix_reserve( size_t s )
{
    if ( s > BUFF_PREFIX_SIZE )
        return;

    data =  data_malloc + PRE_BUFF_SIZE - s;
}

size_t Buffer::getSize()
{
    return size;
}

void Buffer::setSize(size_t s)
{
    size = s;
}

char* Buffer::getDataPtr(size_t offset)
{
    return data + offset;
}

void Buffer::setProducer()
{
    owner = PRODUCER;
    prefix_reserve( 0 );
    setSize( 0 );
}

Buffer* Buffer::isProducer()
{
    return (owner == PRODUCER) ? this : NULL;
}

void Buffer::setConsumer()
{
    owner = CONSUMER;
}

Buffer* Buffer::isConsumer()
{
    return (owner == CONSUMER) ? this : NULL;
}

uint32_t Buffer::getFlags()
{
    return flags;
}

void Buffer::setFlags(uint32_t f)
{
    flags = f;
}

void Buffer::set_frame_data_size( uint32_t fds )
{
    frame_data_size = fds;
}

uint32_t Buffer::get_frame_data_size( void )
{
    return frame_data_size;
}

void Buffer::set_frame_id( int32_t n )
{
    id = n;
}

int32_t Buffer::get_frame_id( void )
{
    return id;
}
