/*!
 * @file fi_http_protocol.cxx
 * @brief all support for managing request/response data for the HTTP protocol
 *
 * Copyright 2018 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include "libwebsockets.h"
#include "fi_http_protocol.h"

// what is the protocol id name for this handler
static const char  *HTTP_PROTOCOL_ID	= "http";

fiHTTPProtocol::fiHTTPProtocol(struct lws_protocols* myProtocolRecPtr)
{
    // not initialized yet
    IAmInitialized = false;

    if (myProtocolRecPtr != (struct lws_protocols*)NULL)
    {
        // setup the protocol data for this handler
        myProtocolRecPtr->name = HTTP_PROTOCOL_ID;
        myProtocolRecPtr->callback = lws_callback_http_dummy;
        myProtocolRecPtr->per_session_data_size = 0;
        myProtocolRecPtr->rx_buffer_size = 0;

        // initialized
        IAmInitialized = true;
    }
}

/*! --------------------------------------------------------------------------------
 * @brief Destructs the FI HTTP Protocol Handler Object
 */
fiHTTPProtocol::~fiHTTPProtocol()
{
    IAmInitialized = false;
}

bool fiHTTPProtocol::isReady()
{
    return IAmInitialized;
}
