/*
 * fi_wss_streamer_session.cxx
 *
 *  Created on: Jan 25, 2016
 *      Author: eric
 */

#include <stdio.h>
#include <libwebsockets.h>

#include <fastec/fas_log.h>

#include "fi_streaming_protocol.h"
#include "fi_streaming_session.h"

/**
 * @brief Constructor
 *
 * @param p Points to the protocol instance to which this session will belong
 * @param w Points to the libwebsockets per-session data for this session
 */
StreamingSession::StreamingSession(StreamingProtocol* p, struct lws* w) :
    WssSession(p, w),
    state(OFF),
    frameHeaderFinished(false),
    bytesSoFar(0),
    frame_length(0),
    txQueueNext(NULL),
    txQueuePrev(NULL)
{

}

StreamingSession::~StreamingSession()
{
    // TODO Auto-generated destructor stub
}

void StreamingSession::scheduleWriteCallback()
{
    lws_callback_on_writable(wsi);
}

const char *StreamingSession::decodeState()
{
    const char *rc = "";
    switch (state)
    {
    case NONE:		rc = "NONE"; break;
    case OFF:		rc = "OFF"; break;
    case ON_REQ:	rc = "ON_REQ"; break;
    case ON_FRAME:	rc = "ON_FRAME"; break;
    case ON_IDLE:	rc = "ON_IDLE"; break;
    case OFF_REQ:	rc = "OFF_REQ"; break;
    default:
	break;
    }
    return rc;
}

StreamingSession::SessionState StreamingSession::getState()
{
    return state;
}

void StreamingSession::setState(StreamingSession::SessionState s)
{
    state = s;
}

int StreamingSession::getBytesSoFar()
{
    return bytesSoFar;
}

void StreamingSession::setBytesSoFar(int n)
{
    bytesSoFar = n;
}

void StreamingSession::set_frame_length(size_t l)
{
    frame_length = l;
}

size_t StreamingSession::get_frame_length()
{
    return frame_length;
}

void StreamingSession::setTxQueueLinks(StreamingSession* p, StreamingSession* n)
{
    txQueuePrev = p;
    txQueueNext = n;
}

void StreamingSession::setTxQueuePrev(StreamingSession* p)
{
    txQueuePrev = p;
}

void StreamingSession::setTxQueueNext(StreamingSession* n)
{
    txQueueNext = n;
}

StreamingSession* StreamingSession::getTxQueueNext()
{
    return txQueueNext;
}

StreamingSession* StreamingSession::getTxQueuePrev()
{
    return txQueuePrev;
}
