/*
 * fi_wss_protocol.cxx
 *
 *  Created on: Jan 25, 2016
 *      Author: eric
 */
#include <stdio.h>
#include <stdlib.h>

#include <fastec/fas_log.h>

#include "fi_wss_session.h"
#include "fi_wss_protocol.h"

/**
 * @brief Callback entry point for all protocols
 *
 * This function serves as a trampoline to make calls into
 * protocol member functions on behalf of the libwebsockets server.
 * It also handles the de-multiplxing function required to handle
 * different callback reasons from the libwebsockeets server.
 *
 * @param wsi         Opaque pointer to libwebsockets' per-session data
 * @param reason      Reason that the server made the callback
 * @param user        For WssProtocol classes, this is the WssProtocolSession pointer
 * @param in          Data associated with this callback
 * @param len         Data length
 * @return            0: callback completed successfully; 1: error during callback
 */
static int callback(struct lws *wsi,
                    enum lws_callback_reasons reason,
                    void *user, void *in, size_t len)
{
    const struct lws_protocols *prot = lws_get_protocol(wsi);
    if (prot != NULL)
    {
        WssProtocol *prot_instance = (WssProtocol *)(prot->user);
        if (prot_instance != NULL)
        {
//      LOG("%s: prot_instance=%p, reason=%d", __PRETTY_FUNCTION__, prot_instance, reason);
            switch (reason)
            {
            case LWS_CALLBACK_PROTOCOL_INIT:
                break;

            case LWS_CALLBACK_FILTER_PROTOCOL_CONNECTION:
                return prot_instance->accept(wsi, (WssSession **)user);

            case LWS_CALLBACK_ESTABLISHED:
                return prot_instance->established(wsi, (WssSession **)user);

            case LWS_CALLBACK_CLOSED:
                return prot_instance->closed(wsi, (WssSession **)user);

            case LWS_CALLBACK_PROTOCOL_DESTROY:
                return prot_instance->destroy(wsi, (WssSession *)user);

            case LWS_CALLBACK_SERVER_WRITEABLE:
                return prot_instance->writable(wsi, *(WssSession **)user);

            case LWS_CALLBACK_RECEIVE:
                return prot_instance->receive(wsi, *(WssSession **)user, (uint8_t *)in, len);

            default:
                return 0;
            }
        }
        else
        {
            fi_log(LOG_INFO, "%s: reason=%d, prot_instance == NULL", __PRETTY_FUNCTION__, (int)reason);
        }
    }
    else
    {
        fi_log(LOG_INFO, "%s: reason=%d, prot == NULL", __PRETTY_FUNCTION__, (int)reason);
    }
    return 0;
}

WssProtocol::WssProtocol(struct lws_protocols *p) :
    ctx(0)
{
    p->callback = callback;
    p->per_session_data_size = sizeof(WssSession *);
    p->user = (void *)this;
    p->id = 0;
    p->name = "unspecified";
    p->rx_buffer_size = 0;
    p->tx_packet_size = 0;  // By default, use the same size as rx_buffer_size.
}

WssProtocol::~WssProtocol()
{
    // TODO Auto-generated destructor stub
}

void WssProtocol::set_lws_context(struct lws_context *c)
{
    ctx = c;
}

WssProtocol::iterator WssProtocol::begin()
{
    return sessions.begin();
}
WssProtocol::iterator WssProtocol::end()
{
    return sessions.end();
}

int WssProtocol::accept(struct lws * /*wsi*/, WssSession ** /*s*/)
{
    return 0; // Accept connections unless sub-class overrides
}
