/*!
 * @file fi_wss_utils.cxx
 * @brief implements liveview support for communicating with the camera server.
 *
 * Copyright 2015 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <stdint.h>

#include <string>

#include "libwebsockets.h"

#include "fi_wss_utils.h"

// NOTE:  This file was primarily created when pulling out useful debugging or
// other utilities from the testserver.c (open source of libwebsockets examples)
// some of the protocol handlers want to use this for debugging purposes.

/*! --------------------------------------------------------------------------------
 * @brief WSS Protocols Utility Function to log WSI Handshake Info
 *
 * Called to provide support to debug important information from the wsi during
 * protocol sessions.  Copied AS IS with exception of using lwl_debug for the
 * logging mechanism instead of fprintf to stderr\
 *
 * Logs the Information as WSS information if debug_level is set
 *  --------------------------------------------------------------------------------*/
void dump_handshake_info(struct lws *wsi)
{
    int n = 0;
    char buf[256];
    const unsigned char *c;

    do
    {
        c = lws_token_to_string((enum lws_token_indexes) n);
        if (!c)
        {
            n++;
            continue;
        }

        if (!lws_hdr_total_length(wsi, (enum lws_token_indexes)n))
        {
            n++;
            continue;
        }

        lws_hdr_copy(wsi, buf, sizeof buf, (enum lws_token_indexes)n);

        lwsl_debug("    %s = %s\n", (char *)c, buf);
        n++;
    }
    while (c);
}

// end of fi_wss_utils.cxx
